# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import os
import shutil
import tempfile
import sys

try:
    import unittest2 as unittest
except:
    import unittest

from os.path import join, relpath
from contextlib import contextmanager
from mock import patch, MagicMock, ANY
from textwrap import dedent

from bumpr.config import Config, ObjectDict
from bumpr.helpers import execute


@contextmanager
def workspace(module_name='fake', version='1.2.3.dev'):
    root = tempfile.mkdtemp(module_name)
    module_filename = join(root, '{0}.py'.format(module_name))
    module_content =  '''\
        # -*- coding: utf-8 -*-

        __version__ = '{version}'
        '''
    readme_filename = join(root, 'README')
    readme_content = '''\
        README

        Version: {version}
        Lorem ipsum dolor sit amet, consectetur adipisicing elit.
        Non, ad, facilis, vel voluptas fugiat sit debitis iusto
        numquam quasi aliquid cum quod laborum assumenda quia
        '''

    for filename, content in ((module_filename, module_content), (readme_filename, readme_content)):
        with open(filename, 'wb') as f:
            content = dedent(content).format(version=version)
            f.write(content.encode('utf8'))

    cwd = os.getcwd()
    os.chdir(root)

    yield ObjectDict({
        'root': root,
        'module': module_filename,
        'readme': readme_filename,
    })

    os.chdir(cwd)
    shutil.rmtree(root)
    if root in sys.path:
        sys.path.remove(root)


class WorkspaceTest(unittest.TestCase):
    def test_behavior(self):
        initial_dir = os.getcwd()
        with workspace('fake') as wksp:
            self.assertNotEqual(os.getcwd(), initial_dir)
            self.assertEqual(wksp.root, os.getcwd())
            self.assertEqual(wksp.module, join(wksp.root, 'fake.py'))
            self.assertEqual(wksp.readme, join(wksp.root, 'README'))
        self.assertEqual(os.getcwd(), initial_dir)



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import codecs
try:
    import unittest2 as unittest
except:
    import unittest

from datetime import datetime
from mock import patch, MagicMock, ANY
from textwrap import dedent

from bumpr.config import ObjectDict
from bumpr.helpers import BumprError
from bumpr.hooks import ReadTheDocHook, CommandsHook, ChangelogHook
from bumpr.version import Version

from tests.test_tools import workspace

class ReadTheDocHookTest(unittest.TestCase):
    def setUp(self):
        self.releaser = MagicMock()
        self.releaser.version = Version.parse('1.2.3')
        self.releaser.config.__getitem__.return_value = ObjectDict({
            'id': 'fake',
            'url': 'http://{id}.somewhere.io/{tag}',
            'bump': '{version}',
            'prepare': 'latest',
        })
        self.hook = ReadTheDocHook(self.releaser)

    def test_bump(self):
        replacements = []
        self.hook.bump(replacements)
        self.assertEqual(replacements, [('http://fake.somewhere.io/latest', 'http://fake.somewhere.io/1.2.3')])

    def test_prepare(self):
        replacements = []
        self.hook.prepare(replacements)
        self.assertEqual(replacements, [('http://fake.somewhere.io/1.2.3', 'http://fake.somewhere.io/latest')])


@patch('bumpr.hooks.execute')
class CommandsHookTest(unittest.TestCase):
    def setUp(self):
        self.releaser = MagicMock()
        self.releaser.version = Version.parse('1.2.3')
        self.releaser.config.__getitem__.return_value = ObjectDict({
            'bump': 'bump command',
            'prepare': 'prepare command',
        })
        self.releaser.config.verbose = False
        self.releaser.config.dryrun = False
        self.hook = CommandsHook(self.releaser)

    def test_bump(self, execute):
        self.hook.bump([])
        execute.assert_called_once_with('bump command', replacements=ANY, verbose=ANY, dryrun=False)

    def test_prepare(self, execute):
        self.hook.prepare([])
        execute.assert_called_once_with('prepare command', replacements=ANY, verbose=ANY, dryrun=False)

    def test_bump_dryrun(self, execute):
        self.hook.dryrun = True
        self.hook.bump([])
        execute.assert_called_once_with('bump command', replacements=ANY, verbose=ANY, dryrun=True)

    def test_prepare_dryrun(self, execute):
        self.hook.dryrun = True
        self.hook.prepare([])
        execute.assert_called_once_with('prepare command', replacements=ANY, verbose=ANY, dryrun=True)


class ChangelogHookTest(unittest.TestCase):
    def setUp(self):
        self.releaser = MagicMock()
        self.releaser.prev_version = Version.parse('1.2.3.dev')
        self.releaser.version = Version.parse('1.2.3')
        self.releaser.next_version = Version.parse('1.2.4.dev')
        self.releaser.timestamp = datetime.now()
        self.releaser.config.__getitem__.return_value = ObjectDict({})
        self.releaser.config.encoding = 'utf8'
        self.releaser.config.verbose = False
        self.releaser.config.dryrun = False

    def test_validate_no_file(self):
        with self.assertRaises(BumprError):
            hook = ChangelogHook(self.releaser)

    def test_validate_file_does_not_exists(self):
        with workspace():
            self.releaser.config.__getitem__.return_value = ObjectDict({'file': 'changelog'})
            with self.assertRaises(BumprError):
                hook = ChangelogHook(self.releaser)

    def test_validate(self):
        with workspace() as wksp:
            with codecs.open('changelog', 'wb', encoding='utf8') as f:
                f.write('')
            self.releaser.config.__getitem__.return_value = ObjectDict({'file': 'changelog'})
            ChangelogHook(self.releaser)

    def test_bump(self):
        content = dedent('''\
            Dev
            ###

            - some changes
        ''')

        with workspace() as wksp:
            with codecs.open('changelog', 'wb', encoding='utf8') as f:
                f.write(content)

            self.releaser.config.__getitem__.return_value = ObjectDict({
                'file': 'changelog',
                'separator': '#',
                'bump': '{version} {date:%Y-%m-%d}',
                'prepare': 'Dev',
                'empty': 'Empty',
            })

            hook = ChangelogHook(self.releaser)
            hook.bump([])

            expected = dedent('''\
                1.2.3 {0:%Y-%m-%d}
                ################

                - some changes
            ''').format(self.releaser.timestamp)

            self.releaser.perform.assert_called_once_with('changelog', content, expected)

    def test_prepare(self):
        content = dedent('''\
            1.2.3 {0:%Y-%m-%d}
            ################

            - some changes
        ''').format(self.releaser.timestamp)

        with workspace() as wksp:
            with codecs.open('changelog', 'wb', encoding='utf8') as f:
                f.write(content)

            self.releaser.config.__getitem__.return_value = ObjectDict({
                'file': 'changelog',
                'separator': '#',
                'bump': '{version} {date:%Y-%m-%d}',
                'prepare': 'Dev',
                'empty': 'Empty',
            })

            hook = ChangelogHook(self.releaser)
            hook.prepare([])

            expected = dedent('''\
                Dev
                ###

                - Empty

                1.2.3 {0:%Y-%m-%d}
                ################

                - some changes
            ''').format(self.releaser.timestamp)

            self.releaser.perform.assert_called_once_with('changelog', content, expected)



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import sys

try:
    import unittest2 as unittest
except:
    import unittest

from mock import patch, call
from subprocess import CalledProcessError

from bumpr.helpers import execute, BumprError

IS_PY3 = sys.version_info[0] == 3


@patch('bumpr.helpers.check_output')
@patch('subprocess.check_call')
class ExecuteTest(unittest.TestCase):
    def test_execute_quiet(self, check_call, check_output):
        output = 'some output'
        check_output.return_value = output
        self.assertEqual(execute('some command'), output)
        check_output.assert_called_with(['some', 'command'])
        self.assertFalse(check_call.called)

    def test_execute_verbose(self, check_call, check_output):
        execute('some command', verbose=True)
        check_call.assert_called_with(['some', 'command'])
        self.assertFalse(check_output.called)

    def test_execute_array(self, check_call, check_output):
        execute(['some', 'command'])
        check_output.assert_called_with(['some', 'command'])
        self.assertFalse(check_call.called)

    def test_execute_quoted(self, check_call, check_output):
        execute('some command "with quote"')
        check_output.assert_called_with(['some', 'command', 'with quote'])

    def test_execute_format(self, check_call, check_output):
        execute('some command {key}', replacements={'key': 'value'})
        check_output.assert_called_with(['some', 'command', 'value'])

    def test_execute_format_array(self, check_call, check_output):
        execute(['some', 'command', '{key}'], replacements={'key': 'value'})
        check_output.assert_called_with(['some', 'command', 'value'])

    def test_execute_dry(self, check_call, check_output):
        execute('some command', dryrun=True)
        self.assertFalse(check_call.called)
        self.assertFalse(check_output.called)

    def test_execute_multiple(self, check_call, check_output):
        execute('''
            some command
            another command
        ''')

        expected = (
            call(['some', 'command']),
            call(['another', 'command']),
        )
        self.assertSequenceEqual(check_output.call_args_list, expected)

    def test_execute_multiple_array(self, check_call, check_output):
        execute((
            ['some', 'command'],
            ['another', 'command'],
        ))

        expected = (
            call(['some', 'command']),
            call(['another', 'command']),
        )
        self.assertSequenceEqual(check_output.call_args_list, expected)

    def test_execute_error_quiet(self, check_call, check_output):
        error = CalledProcessError(1, 'cmd')
        error.output = 'some output'
        check_output.side_effect = error

        with self.assertRaises(BumprError):
            to_patch = '{0}.print'.format('builtins' if IS_PY3 else '__builtin__')
            with patch(to_patch):
                execute('some failed command')

    def test_execute_error_verbose(self, check_call, check_output):
        check_call.side_effect = CalledProcessError(1, 'cmd')

        with self.assertRaises(BumprError):
            execute('some failed command', verbose=True)



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import io
import sys

try:
    import unittest2 as unittest
except:
    import unittest

from contextlib import contextmanager
from copy import deepcopy
from mock import patch
from textwrap import dedent

from bumpr.config import DEFAULTS, Config, ValidationError, __name__ as config_module_name
from bumpr.version import Version
from bumpr.hooks import HOOKS, ReadTheDocHook

if sys.version_info[0] == 3:
    unicode = str  # pylint: disable=W0622,C0103


@contextmanager
def mock_ini(data):
    open_name = '{0}.open'.format(config_module_name)
    with patch(open_name, return_value=io.StringIO(unicode(dedent(data))), create=True) as mock:
        yield mock


class ConfigTest(unittest.TestCase):
    maxDiff = None

    def test_defaults(self):
        '''It should initialize with default values'''
        config = Config()
        expected = deepcopy(DEFAULTS)
        for hook in HOOKS:
            expected[hook.key] = False
        self.assertDictEqual(config, expected)

    def test_from_dict(self):
        '''It can take a dictionnay as parameter but keeps defaults'''
        config_dict = {
            'module': 'test_module',
            'attribute': 'VERSION',
            'commit': True,
            'tag': True,
            'dryrun': True,
            'files': ['anyfile.py'],
            'bump': {
                'suffix': 'final',
                'message': 'Version {version}',
            },
            'prepare': {
                'part': 'minor',
                'suffix': 'dev',
                'message': 'Update to version {version}',
            },
        }
        config = Config(config_dict)
        expected = deepcopy(DEFAULTS)
        for key, value in config_dict.items():
            if isinstance(value, dict):
                for nested_key, nested_value in value.items():
                    expected[key][nested_key] = nested_value
            else:
                expected[key] = value
        for hook in HOOKS:
            expected[hook.key] = False
        self.assertDictEqual(config, expected)

    def test_from_dict_with_hook(self):
        '''It should take defaults from hooks if present and set it to false if not'''
        tested_hook = ReadTheDocHook
        config_dict = {
            tested_hook.key: {
                'bump': 'test'
            }
        }

        expected = deepcopy(DEFAULTS)
        for hook in HOOKS:
            if hook is tested_hook:
                expected[hook.key] = hook.defaults
                expected[hook.key]['bump'] = 'test'
            else:
                expected[hook.key] = False

        config = Config(config_dict)
        self.assertDictEqual(config, expected)

    def test_override_from_config(self):
        bumprrc = '''\
        [bumpr]
        file = test.py
        files = README
        [bump]
        message = test
        '''

        expected = deepcopy(DEFAULTS)
        expected['file'] = 'test.py'
        expected['files'] = ['README']
        expected['bump']['message'] = 'test'
        for hook in HOOKS:
            expected[hook.key] = False

        config = Config()
        with mock_ini(bumprrc) as mock:
            config.override_from_config('test.rc')

        mock.assert_called_once_with('test.rc')
        self.assertDictEqual(config, expected)

    def test_override_hook_from_config(self):
        tested_hook = ReadTheDocHook
        bumprrc = '''\
        [{0}]
        bump = test
        '''.format(tested_hook.key)

        expected = deepcopy(DEFAULTS)
        for hook in HOOKS:
            if hook is tested_hook:
                expected[hook.key] = hook.defaults
                expected[hook.key]['bump'] = 'test'
            else:
                expected[hook.key] = False

        config = Config()
        with mock_ini(bumprrc) as mock:
            config.override_from_config('test.rc')

        mock.assert_called_once_with('test.rc')
        self.assertDictEqual(config, expected)

    def test_override_from_args(self):
        config = Config.parse_args(['test.py', '-M', '-v', '-s', 'test-suffix', '-c', 'fake'])

        expected = deepcopy(DEFAULTS)
        expected['file'] = 'test.py'
        expected['bump']['part'] = Version.MAJOR
        expected['bump']['suffix'] = 'test-suffix'
        expected['verbose'] = True
        for hook in HOOKS:
            expected[hook.key] = False

        self.assertDictEqual(config, expected)

    def test_override_args_keeps_config_values(self):
        bumprrc = '''\
        [bumpr]
        files = README
        [bump]
        message = test
        [prepare]
        part = minor
        '''

        with mock_ini(bumprrc):
            with patch('bumpr.config.exists', return_value=True):
                config = Config.parse_args(['test.py', '-M', '-v', '-s', 'test-suffix', '-c', 'test.rc'])

        expected = deepcopy(DEFAULTS)
        expected['file'] = 'test.py'
        expected['bump']['part'] = Version.MAJOR
        expected['bump']['suffix'] = 'test-suffix'
        expected['verbose'] = True

        expected['files'] = ['README']
        expected['bump']['message'] = 'test'
        expected['prepare']['part'] = Version.MINOR

        for hook in HOOKS:
            expected[hook.key] = False

        self.assertDictEqual(config, expected)

    def test_validate(self):
        config = Config({'file': 'version.py'})
        config.validate()

    def test_validate_file_missing(self):
        config = Config()
        with self.assertRaises(ValidationError):
            config.validate()



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import os
try:
    import unittest2 as unittest
except:
    import unittest

from mock import patch

from tests.test_tools import workspace

from bumpr.vcs import BaseVCS, Git, Mercurial, Bazaar
from bumpr.helpers import BumprError


class BaseVCSTest(unittest.TestCase):
    def test_execute_verbose(self):
        vcs = BaseVCS(verbose=True)
        with patch('bumpr.vcs.execute') as execute:
            vcs.execute('cmd arg')
            execute.assert_called_with('cmd arg', verbose=True)

    def test_execute_quiet(self):
        vcs = BaseVCS(verbose=False)
        with patch('bumpr.vcs.execute') as execute:
            vcs.execute('cmd arg')
            execute.assert_called_with('cmd arg', verbose=False)


class GitTest(unittest.TestCase):
    def test_validate_ok(self):
        with workspace('git'):
            os.mkdir('.git')
            git = Git()

            with patch('bumpr.vcs.execute') as execute:
                execute.return_value = '?? new.py'
                git.validate()
                execute.assert_called_with('git status --porcelain', verbose=False)

    def test_validate_ko_not_git(self):
        with workspace('git'):
            git = Git()

            with patch('bumpr.vcs.execute') as execute:
                with self.assertRaises(BumprError):
                    git.validate()
                self.assertFalse(execute.called)

    def test_validate_ko_not_clean(self):
        with workspace('git'):
            os.mkdir('.git')
            git = Git()

            with patch('bumpr.vcs.execute') as execute:
                execute.return_value = '\n'.join((' M modified.py', '?? new.py'))
                with self.assertRaises(BumprError):
                    git.validate()
                execute.assert_called_with('git status --porcelain', verbose=False)

    def test_tag(self):
        git = Git()

        with patch.object(git, 'execute') as execute:
            git.tag('fake')
            execute.assert_called_with(['git', 'tag', 'fake'])

    def test_commit(self):
        git = Git()

        with patch.object(git, 'execute') as execute:
            git.commit('message')
            execute.assert_called_with(['git', 'commit', '-am', 'message'])


class MercurialTest(unittest.TestCase):
    def test_validate_ok(self):
        with workspace('mercurial'):
            os.mkdir('.hg')
            mercurial = Mercurial()

            with patch('bumpr.vcs.execute') as execute:
                execute.return_value = '?? new.py'
                mercurial.validate()
                execute.assert_called_with('hg status -mard', verbose=False)

    def test_validate_ko_not_mercurial(self):
        with workspace('mercurial'):
            mercurial = Mercurial()

            with patch('bumpr.vcs.execute') as execute:
                with self.assertRaises(BumprError):
                    mercurial.validate()
                self.assertFalse(execute.called)

    def test_validate_ko_not_clean(self):
        with workspace('mercurial'):
            os.mkdir('.hg')
            mercurial = Mercurial()

            with patch('bumpr.vcs.execute') as execute:
                execute.return_value = '\n'.join((' M modified.py', '?? new.py'))
                with self.assertRaises(BumprError):
                    mercurial.validate()
                execute.assert_called_with('hg status -mard', verbose=False)

    def test_tag(self):
        mercurial = Mercurial()

        with patch.object(mercurial, 'execute') as execute:
            mercurial.tag('fake')
            execute.assert_called_with(['hg', 'tag', 'fake'])

    def test_commit(self):
        mercurial = Mercurial()

        with patch.object(mercurial, 'execute') as execute:
            mercurial.commit('message')
            execute.assert_called_with(['hg', 'commit', '-A', '-m', 'message'])


class BazaarTest(unittest.TestCase):
    def test_validate_ok(self):
        with workspace('bazaar'):
            os.mkdir('.bzr')
            bazaar = Bazaar()

            with patch('bumpr.vcs.execute') as execute:
                execute.return_value = '? new.py'
                bazaar.validate()
                execute.assert_called_with('bzr status --short', verbose=False)

    def test_validate_ko_not_bazaar(self):
        with workspace('bazaar'):
            bazaar = Bazaar()

            with patch('bumpr.vcs.execute') as execute:
                with self.assertRaises(BumprError):
                    bazaar.validate()
                self.assertFalse(execute.called)

    def test_validate_ko_not_clean(self):
        with workspace('bazaar'):
            os.mkdir('.bzr')
            bazaar = Bazaar()

            with patch('bumpr.vcs.execute') as execute:
                execute.return_value = '\n'.join((' M modified.py', '? new.py'))
                with self.assertRaises(BumprError):
                    bazaar.validate()
                execute.assert_called_with('bzr status --short', verbose=False)

    def test_tag(self):
        bazaar = Bazaar()

        with patch.object(bazaar, 'execute') as execute:
            bazaar.tag('fake')
            execute.assert_called_with(['bzr', 'tag', 'fake'])

    def test_commit(self):
        bazaar = Bazaar()

        with patch.object(bazaar, 'execute') as execute:
            bazaar.commit('message')
            execute.assert_called_with(['bzr', 'commit', '-m', 'message'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from nose.plugins import Plugin

from bumpr.log import declare


class DryRunLoggerPlugin(Plugin):
    name = 'dryrun-logger'
    enabled = True

    def configure(self, options, conf):
        pass  # always on

    def begin(self):
        declare()  # Fix tests with custom log level



# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals

import os
import shutil
import tempfile
import sys

try:
    import unittest2 as unittest
except:
    import unittest

from os.path import join, relpath
from contextlib import contextmanager
from mock import patch, MagicMock, ANY
from textwrap import dedent

from tests.test_tools import workspace

from bumpr.config import Config, ObjectDict
from bumpr.helpers import execute
from bumpr.releaser import Releaser, HOOKS
from bumpr.version import Version



class ReleaserTest(unittest.TestCase):
    def test_constructor(self):
        config = Config({
            'file': 'fake.py'
        })
        with workspace('fake', '1.2.3.dev') as wksp:
            releaser = Releaser(config)

        self.assertIsInstance(releaser.prev_version, Version)
        self.assertEqual(str(releaser.prev_version), '1.2.3.dev')

        self.assertIsInstance(releaser.version, Version)
        self.assertIsInstance(releaser.next_version, Version)

        self.assertIsNone(releaser.timestamp)

        self.assertFalse(hasattr(releaser, 'vcs'))
        self.assertFalse(hasattr(releaser, 'diffs'))

        self.assertEqual(releaser.hooks, [])

    def test_constructor_with_hooks(self):
        config = Config({
            'file': 'fake.py'
        })
        hooks = []
        for i in range(3):
            key = 'hook{0}'.format(i)
            config[key] = True
            mock = MagicMock()
            mock.key = key
            hooks.append(mock)

        with workspace('fake', '1.2.3.dev') as wksp:
            with patch('bumpr.releaser.HOOKS', hooks) as mock:
                releaser = Releaser(config)
                for hook in hooks:
                    hook.assert_called_with(releaser)

    def test_test(self):
        config = Config({
            'file': 'fake.py',
            'tests': 'test command',
        })
        with workspace('fake'):
            releaser = Releaser(config)

        with patch('bumpr.releaser.execute') as execute:
            releaser.test()
            execute.assert_called_with('test command', replacements=ANY, dryrun=ANY, verbose=ANY)

    def test_publish(self):
        config = Config({
            'file': 'fake.py',
            'publish': 'publish command',
        })

        with workspace('fake'):
            releaser = Releaser(config)

        with patch('bumpr.releaser.execute') as execute:
            releaser.publish()
            execute.assert_called_with('publish command', replacements=ANY, dryrun=ANY, verbose=ANY)

    def test_clean(self):
        config = Config({
            'file': 'fake.py',
            'clean': 'clean command',
        })
        with workspace('fake'):
            releaser = Releaser(config)

        with patch('bumpr.releaser.execute') as execute:
            releaser.clean()
            execute.assert_called_with('clean command', replacements=ANY, dryrun=ANY, verbose=ANY)

    def test_commit(self):
        config = Config({'file': 'fake.py', 'vcs': 'fake'})
        with workspace('fake') as wksp:
            releaser = Releaser(config)

        with patch.object(releaser, 'vcs') as vcs:
            releaser.commit('message')
            vcs.commit.assert_called_with('message')

    def test_tag(self):
        config = Config({'file': 'fake.py', 'vcs': 'fake'})
        with workspace('fake') as wksp:
            releaser = Releaser(config)

        with patch.object(releaser, 'vcs') as vcs:
            releaser.tag()
            vcs.tag.assert_called_with(str(releaser.version))

    def test_release_wihtout_vcs_or_commands(self):
        with workspace('fake', '1.2.3.dev') as wksp:
            config = Config({'file': 'fake.py', 'files': [wksp.readme]})
            releaser = Releaser(config)
            with patch('bumpr.releaser.execute') as execute:
                with patch.object(releaser, 'commit') as commit:
                    releaser.release()
                    self.assertFalse(execute.called)
                    self.assertFalse(commit.called)

            for filename in wksp.module, wksp.readme:
                with open(filename) as f:
                    content = f.read()
                    self.assertIn('1.2.3', content)
                    self.assertNotIn('1.2.3.dev', content)

    def test_bump(self):
        with workspace('fake', '1.2.3.dev') as wksp:
            config = Config({'file': 'fake.py', 'files': [wksp.readme]})
            releaser = Releaser(config)
            with patch.object(releaser, 'commit') as commit:
                with patch.object(releaser, 'tag') as tag:
                    releaser.bump()
                    self.assertFalse(commit.called)
                    self.assertFalse(tag.called)

            for filename in wksp.module, wksp.readme:
                with open(filename) as f:
                    content = f.read()
                    self.assertIn('1.2.3', content)
                    self.assertNotIn('1.2.3.dev', content)

    def test_bump_vcs(self):
        with workspace('fake', '1.2.3.dev') as wksp:
            config = Config({
                'file': 'fake.py',
                'files': [wksp.readme],
                'vcs': 'fake',
            })
            releaser = Releaser(config)
            with patch.object(releaser, 'commit') as commit:
                with patch.object(releaser, 'tag') as tag:
                    releaser.bump()
                    self.assertEqual(commit.call_count, 1)
                    self.assertTrue(tag.called)

            for filename in wksp.module, wksp.readme:
                with open(filename) as f:
                    content = f.read()
                    self.assertIn('1.2.3', content)
                    self.assertNotIn('1.2.3.dev', content)

    def test_release_dryrun(self):
        with workspace('fake', '1.2.3.dev') as wksp:
            config = Config({
                'file': 'fake.py',
                'files': [wksp.readme],
                'vcs': 'fake',
                'dryrun': True,
            })
            releaser = Releaser(config)
            with patch('bumpr.releaser.execute') as execute:
                with patch.object(releaser, 'vcs') as vcs:
                    releaser.release()
                    self.assertFalse(execute.called)
                    self.assertFalse(vcs.commit.called)
                    self.assertFalse(vcs.tag.called)

            for filename in wksp.module, wksp.readme:
                with open(filename) as f:
                    content = f.read()
                    self.assertIn('1.2.3.dev', content)
                    self.assertNotIn('1.2.4', content)

    def test_prepare(self):
        with workspace('fake', '1.2.3') as wksp:
            config = Config({
                'file': 'fake.py',
                'files': [wksp.readme],
                'prepare': {
                    'part': Version.PATCH,
                    'suffix': 'dev',
                }
            })
            releaser = Releaser(config)
            with patch.object(releaser, 'commit') as commit:
                with patch.object(releaser, 'tag') as tag:
                    releaser.prepare()
                    self.assertFalse(commit.called)
                    self.assertFalse(tag.called)

            for filename in wksp.module, wksp.readme:
                with open(filename) as f:
                    content = f.read()
                    self.assertIn('1.2.4.dev', content)
                    self.assertNotIn('1.2.3', content)

    def test_prepare_vcs(self):
        with workspace('fake', '1.2.3') as wksp:
            config = Config({
                'file': 'fake.py',
                'files': [wksp.readme],
                'vcs': 'fake',
                'prepare': {
                    'part': Version.PATCH,
                    'suffix': 'dev',
                }
            })
            releaser = Releaser(config)
            with patch.object(releaser, 'commit') as commit:
                with patch.object(releaser, 'tag') as tag:
                    releaser.prepare()
                    self.assertEqual(commit.call_count, 1)
                    self.assertFalse(tag.called)

            for filename in wksp.module, wksp.readme:
                with open(filename) as f:
                    content = f.read()
                    self.assertIn('1.2.4.dev', content)
                    self.assertNotIn('1.2.3', content)



# -*- coding: utf-8 -*-
import sys

try:
    import unittest2 as unittest
except:
    import unittest

from bumpr.version import  Version


class VersionTest(unittest.TestCase):
    def test_default_constructor(self):
        version = Version()

        self.assertEqual(version.major, 0)
        self.assertEqual(version.minor, 0)
        self.assertEqual(version.patch, 0)
        self.assertEqual(version.suffix, None)

    def test_constructor(self):
        version = Version(1, suffix='dev')

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 0)
        self.assertEqual(version.patch, 0)
        self.assertEqual(version.suffix, 'dev')

    def test_bump_major(self):
        version = Version(1, 2, 3, 'dev')

        version.bump(Version.MAJOR)

        self.assertEqual(version.major, 2)
        self.assertEqual(version.minor, 0)
        self.assertEqual(version.patch, 0)
        self.assertEqual(version.suffix, None)

    def test_bump_minor(self):
        version = Version(1, 2, 3, 'dev')

        version.bump(Version.MINOR)

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 3)
        self.assertEqual(version.patch, 0)
        self.assertEqual(version.suffix, None)

    def test_bump_patch(self):
        version = Version(1, 2, 3, 'dev')

        version.bump(Version.PATCH)

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 4)
        self.assertEqual(version.suffix, None)

    def test_bump_suffix(self):
        version = Version(1, 2, 3, 'dev')

        version.bump(suffix='rc1')

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 3)
        self.assertEqual(version.suffix, 'rc1')

    def test_bump_unsuffix(self):
        version = Version(1, 2, 3, 'dev')

        version.bump(unsuffix=True)

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 3)
        self.assertEqual(version.suffix, None)

    def test_bump_suffix_override_unsuffix(self):
        version = Version(1, 2, 3, 'dev')

        version.bump(suffix='rc1', unsuffix=True)

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 3)
        self.assertEqual(version.suffix, 'rc1')

    def test_bump_no_effect(self):
        version = Version(1, 2, 3, 'dev')

        version.bump(unsuffix=False)

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 3)
        self.assertEqual(version.suffix, 'dev')

    def test_copy(self):
        version = Version(1, 2, 3, 'dev').copy()

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 3)
        self.assertEqual(version.suffix, 'dev')

    def test_copy_bump(self):
        version = Version(1, 2, 3, 'dev').copy(part=Version.MINOR, unsuffix=True)

        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 3)
        self.assertEqual(version.patch, 0)
        self.assertEqual(version.suffix, None)


    def test_parse(self):
        version = Version.parse('1.2.3')

        self.assertIsNotNone(version)
        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 3)
        self.assertEqual(version.suffix, None)

    def test_parse_with_suffix(self):
        version = Version.parse('1.2.3.rc4')

        self.assertIsNotNone(version)
        self.assertEqual(version.major, 1)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.patch, 3)
        self.assertEqual(version.suffix, 'rc4')
